#include "global.h"

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>

#define ADC_CHANNELS		5					// Anzahl der zu messenden ADC-Kanle 

uint8_t volatile button, button_1, button_2, temp_button;
uint16_t volatile equal_counter;

uint16_t volatile adc_value [ADC_CHANNELS];			// Feld fr die Wandlungsergebnisse des ADC = ADC-Ergebnis-Speicher
uint8_t volatile adc_channel[] = {7,7,1,2,0};		// Kanalabfolge fr das Auslesen der ADC-Werte
uint8_t volatile adc_index;							// Index fr Kanalwahl und ADC-Speicher



/*
Leseabfolge fr den ADC:
-------------------------

adc_channel[] = {Tasten , Tasten , Quellenspannung , Elektrodenspg. , Elektrodenstrom ,....}

adc_index[0,1] sind fix fr die Tastenabfrage
adc_index[2] fr Quellenspg.
adc_index[3] fr Elektrodenspg.
adc_index[4] fr Elektrodenstrom
adc_index[>4] fr weitere Messkanle
*/



/***************************************************************************************
*
*Funktionsname:		adc_init
*
*Rckgabe:			keine
*
*Parameter:			keine
*
*Aufgabe:			Initialisierung des AD-Converters
*
*getestet: 
*
***************************************************************************************/

void adc_init(void)
{
	button = 0;					// Initialisierung der Variablen
	button_1 = 0;
	button_2 = 0;
	equal_counter = 0;
	
		
	ADMUX &= ~( (1<<REFS1) | (1<<REFS0) ); 	
	//Externe Referenzspannung nutzen -> REFS1 uns REFS0 auf 0 setzen
										
	ADCSRA |= (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
	/* Maximale Wandlerfrequenz = 200kHz !
	Dadurch ergibt sich ein Teilungsfaktor von F_CPU/200kHz = 80 bei 16Mhz Quarz
	Nchster Faktor ist 128
	Somit ergibt sich eine Wandlerfrequenz von 125kHz
	ADPS2, ADPS1, ADPS0 auf 1 setzen*/
	
	ADCSRA |= (1<<ADIE);
	// Interrupt, wenn Wandlung beendet -> ADIE=1
	
	ADCSRA |= (1<<ADEN);	//ADC durch ADEN=1 aktivieren
	
	adc_index = 0;			// adc_index fr 1. Wandlung auf 0
	
	ADMUX = ( (ADMUX & ~(0x1F)) | (adc_channel[adc_index] & 0x1F) ); // 1. Kanal  einstellen	

/* Kanal  whlen, ohne andere Bits zu verndern

	 0bxxx00000		ADMUX & ~(0x1F) -> Channel Bits auf 0 setzen
	 0bxxx00001		1 & 0x1F  (=channel=1)
	____________	oder - Verknpfung
	 0bxxx00001									*/

	//ADCSRA |= (1<<ADSC);	//1. ADC-Wandlung anstoen
}


/*****************************************************************************************
*							Interrupt Service Routine: ADC_vect
*
*
*
*
*Aufgabe:	Der Interrupt wird ausgefhrt, wenn eine ADC-Wandlung beendet ist
*			
*
*****************************************************************************************/



ISR (ADC_vect)
{
	
	ADMUX = ( (ADMUX & ~(0x1F)) | 0x1F );	// ADC auf GND legen (discharge S&H Cap)
	
	adc_value[adc_index] = ADCW;	// Wandlungsergebins auslesen und speichern
	

// ******** ANFANG Tastenabfrage ******************************	
	
	if (adc_index == 0)							// 1. Wandlungsergebnis fr Tasten
		button_1 = read_LUT(adc_value[0]);		// Taste zuweisen und speichern
			
	
	
	if (adc_index == 1)							// 2tes Wandlungsergebnis fr Tasten
	{	
		button_2 = read_LUT(adc_value[1]);		// Taste zuweisen und speichern
		
		if (button_1 == button_2)		// wenn 2 gleiche Tasten-Nr. hintereinander
		{	
			temp_button = button_2;		// Tastennummer sichern
			
			if(++equal_counter == 65000)	// Prellzhler erhhen und vor berlauf schtzen
				equal_counter = 0;
		}
	}
	
	
	if ( (equal_counter < 5) && (button_1 == 0) )	// Taste losgelassen, kein gltiger Tastendruck	
	{
		button = 0;
		equal_counter = 0;
	}
	
	
	if ( (equal_counter > 5) && (button_1 == 0) )	// Taste losgelassen, gltiger Tastendruck	
	{
		button = temp_button;						// Tasten-Nr. ausgeben
		equal_counter = 0;							// Prellzhler rcksetzen
	}
	
// ***** ENDE Tastenabfrage	********

	
	if( ++adc_index == ADC_CHANNELS)	// Index erhhen und wenn ntig rcksetzen
		adc_index = 0;
		
	ADMUX = ( (ADMUX & ~(0x1F)) | (adc_channel[adc_index] & 0x1F) ); // nchsten Kanal  einstellen	
	
	//ADCSRA |= (1<<ADSC);	//nchste ADC-Wandlung anstoen
	
}