/****************************************************************************************
*
*					timer1.c
*
*
*
*
* Basierend auf dem Code von Peter Dannegger
*****************************************************************************************/

#include "global.h"

#ifndef XTAL
#define XTAL 16000000L					// Quarzfrequenz in Hertz
#endif

#define SOFT_PRESCALER 125L				// Software Teiler

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>

/************************ globale Variablen *****************************************/

uint8_t volatile soft_prescaler;
uint8_t volatile time_puls;



/****************************************************************************************
*
* Funktionsname:		init_timer1
*
* Rckgabe:				--
*
*Parameter:				--
*
*Aufgabe:				Initialisiert Timer1: CTC-Mode, Prescaler=1, 
*						und setzt Vergleichswert OCR1A
*						Initialisierung von Sekundenpuls
*****************************************************************************************/

void timer1_init(void)
{
	time_puls = 0;							// Variablen initialisieren
		
	TCCR1B |= (1<<WGM12) | (1<<CS10);			// CTC-Mode, Prescaler=1
	OCR1A = XTAL / SOFT_PRESCALER - 1;			// Vergleichswert setzen
	TCNT1 = 0;									// Zhlregister initialisieren
	soft_prescaler = (uint8_t) SOFT_PRESCALER;	// Software Teiler setzen
	TIMSK |= (1<<OCIE1A);						// Interrupt fr Comparewert A ein
}




/*****************************************************************************************
*							Interrupt Service Routine: TIMER1_COMPA
*
*
*
*
*Aufgabe:	Der Vergleichswert von Timer1 (OCR1A) ist so gewhlt, da er alle 4msec
*			ereicht wird und auch zum Entprellen von Tasten genutzt werden kann.
*			Diese 4msec werden zustzlich durch den Software_Teiler multipliziert, soda
*			ein 500msec-Impuls ( 4msec * 125 = 1/2sec) entsteht.
*
*****************************************************************************************/

ISR (TIMER1_COMPA_vect)
{

	ADCSRA |= (1<<ADSC);	//alle 4msec eine ADC-Wandlung anstoen
	
	
	#if XTAL % SOFT_PRESCALER				// Rest? wenn ja: OCR1A = normaler Vergleichswert
		OCR1A = XTAL / SOFT_PRESCALER - 1;	// Nach dem verlngerten Vergleichswert mu wieder
	#endif									// der normale Comparewert geladen werden

	
		if(--soft_prescaler == 0)			// zhle software_teiler (125) runter bei 0 ist 
											// eine 1/2 sekunde vergangen
		{
			soft_prescaler = (uint8_t) SOFT_PRESCALER;	// setze software_teiler wieder auf 
														// Anfangswert
				
			if (++time_puls == 2)		// time_puls +1, wenn 2 dann Rcksetzen 
				time_puls = 0;

		#if XTAL % SOFT_PRESCALER		// Rest? wenn ja: 
										//OCR1A = korrigierter Vergleiswert (nur 1x pro sec)
			OCR1A = (XTAL / SOFT_PRESCALER - 1) + XTAL % SOFT_PRESCALER
		#endif
		}
	
}



