/**************************************************************************************
*									KoSi_Gen.c
*
*
* GGC-Version:			4.5.1
*
* AVR-Studio Version:	5.0.1163
*
* Autor: 				Rick
*
* Getestet:						
*
* Letzte nderung:		
*
*
*
* 
***************************************************************************************/

//********************* Defines *******************************************************

#include "global.h"

#include <avr/io.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/interrupt.h>


//******************** globale Variablen *********************************************

int8_t second_countdown;				// Variablen fr Countdown
int8_t hour_countdown;
int8_t minute_countdown;				

/******************************* Konstanten *******************************************
**************************************************************************************/


#define VAREF	2130L		// Referenzspannung fr AD-Wandler in mV (2,5V -> 2500)

#define K		43L		// Teilerfaktor fr Spannungsteiler zu Spannungsmessung
						// Bsp.: Verh.: 1:25 -> Wert=25
						
#define R_SHUNT 120L	// Widerstandswert fr Shunt (Strommessung-Elektroden) im Ohm


#define K_VOLTAGE1		 (VAREF * K) / 1024 
// Korrekturfaktor1 fr Spg.-Messung, Ergebnis in mV !

#define K_VOLTAGE2		(VAREF / 1024)
// Korrekturfaktor2 fr Spg.-Messung, Ergebnis in mV !

#define K_CURRENT		( (VAREF * 1000000) / (R_SHUNT * 1024) )
// Korrekturfaktor fr Strom-Messung, Ergebnis in uA !




/***************************************************************************************
*
*Funktionsname:		adc_read_avg
*
*Rckgabe:			Mittel der ADC-Messungen
*
*Parameter:			Nummer des ADC-Kanals, Anzahl der Wandlungen
*
*Aufgabe:			Stellt den Kanal des ADC ein, fhrt eine gewnschte 
*					Anzahl von Wandlungen durch und errechnet dessen Mittelwert	
*
*Hinweis:			Anzahl der Messungen darf 34 nicht bersteigen, ansonsten mu  
*					"result" eine 32bit Variable sein! (2^16 / 2^10 = 34,7)
*
* getestet: 
*
***************************************************************************************/
	
uint16_t adc_read_avg (uint8_t channel , uint8_t average)
{

	uint16_t result;
	

	ADMUX = ( (ADMUX & ~(0x1F)) | (channel & 0x1F) );
	/* Kanal whlen, ohne andere Bits zu verndern

	 0bxxx00000		ADMUX & ~(0x1F) -> Channel Bits auf 0 setzen
	 0bxxx00001		1 & 0x1F  (=channel=1)
	____________	oder - Verknpfung
	 0bxxx00001									*/

	 /* Nach der Kanalwahl des ADC wird ein "Dummy Readout" zur Sicherheit durchgefhrt.
		Man startet eine Wandlung, liest den Wert aus und verwirft diesen */
	
	ADCSRA |= (1<<ADSC); // ADSC=1 Wandlung starten
	
	while (ADCSRA & (1<<ADSC)) {}
	//Solange ADSC = 1 ist, ist die Wandlung noch nicht abgeschlossen
		

	result = ADCW;					
	// Ergebnis der Wandlung auslesen, sonst wird Ergebnis der nchsten Wandlung nicht 
	// bernommen

	result = 0;								// Ergebnis verwerfen

	// Durchfhrung mehrerer Messungen und Ergebnise aufaddieren:
	for (uint8_t i=0; i < average; i++)
		{
			ADCSRA |= (1<<ADSC);			// 1ne Wandlung durchfhren
			while (ADCSRA & (1<<ADSC)) {}	// Warten bis Wandlung beendet
			result += ADCW;					// Ergebnisse aufaddieren
		}

	result = result / average;				// Mittelwert bilden
	return result;							// Ergebnis zurck geben
	
}





/***************************************************************************************
*
*Funktionsname:		read_voltage_source
*
*Rckgabe:			Spannung der Quelle in mV
*
*Parameter:			keine
*
*Aufgabe:			Liest die Quellen-Spannung aus und gibt diese zurck.
*
*getestet: 
*
***************************************************************************************/

void read_voltage_source(void)
{
	//uint32_t value_voltage_source;
	
	voltage_source = ( adc_value[2] * K_VOLTAGE1 );
	// Quellenspannung = (ADC-Wert * Korrekturfaktor fr Spg.-Messung
	
	//return value_voltage_source;	// Quellenspg. in mV zurckgeben
	
}






/***************************************************************************************
*
*Funktionsname:		read_voltage_electrodes
*
*Rckgabe:			Spannung der Elektroden in mV
*
*Parameter:			keine
*
*Aufgabe:			Liest die Elektroden-Spannung aus und gibt diese zurck.
*
*getestet: 
*
***************************************************************************************/

void read_voltage_electrodes(void)
{
	cli();
	
	//uint16_t sum_adc_value;	
	
	//sum_adc_value = (adc_value[6] + adc_value[7] + adc_value[8] + adc_value[9]) / 4;
	
	voltage_electrodes = ( (adc_value[3] * K_VOLTAGE1) - (adc_value[4] * K_VOLTAGE2) ) ;
	
	lcd_gotoxy(10,3);							// ADC-Wert fr E-Spg. anzeigen
	lcd_puts("U:");
	utoa (voltage_electrodes , string , 10);
	lcd_puts(string);
	if (adc_value[1] < 100)
		lcd_putc(' ');
	if (adc_value[1] < 10)
		lcd_puts("  ");
	
	sei();																		  
}



/***************************************************************************************
*
*Funktionsname:		read_current_electrodes
*
*Rckgabe:			Strom durch die Elektroden in !! uA !!
*
*Parameter:			keine
*
*Aufgabe:			Liest den Elektroden-Strom aus und gibt diesen zurck.
*
*getestet: 
*
***************************************************************************************/

void read_current_electrodes(void)
{
	cli();
	
	//uint16_t sum_adc_value;	
	
	//sum_adc_value = (adc_value[10] + adc_value[11] + adc_value[12] + adc_value[13]) / 4;
	
	current_electrodes = (adc_value[4] * K_CURRENT);
	// Elektrodenstrom [uA]! = ADC-Wert * Korrekturfaktor fr Strommessung
	
	lcd_gotoxy(0,3);							// ADC-Wert fr E-Strom anzeigen
	lcd_puts("I:");
	utoa (adc_value[4] , string , 10);
	lcd_puts(string);
	if (adc_value[1] < 100)
		lcd_putc(' ');
	if (adc_value[1] < 10)
		lcd_puts("  ");
		
	sei();
}


/***************************************************************************************
*
*Funktionsname:		read_LUT (Look Up Table)
*
*Rckgabe:			Tastennummer (1 bis 5), 0 fr keine Taste gedrckt
*
*Parameter:			ADC-Wert
*
*Aufgabe:			Ermittelt zum ausgelesenen ADC-Wert die gedrckte Taste
*
*
* getestet: 
*
***************************************************************************************/

uint8_t read_LUT(uint16_t adc_val_button)
{
	cli();
	
	uint8_t button;
			
	// ************* LUT fr AREF = 2,13V ************************
		
		
		if(adc_val_button <= 710) button = 1;	// Taster 1
		if(adc_val_button <= 665) button = 2;
		if(adc_val_button <= 535) button = 3;
		if(adc_val_button <= 404) button = 4;
		if(adc_val_button <= 270) button = 5;	// Taster 5
		if(adc_val_button <=  129) button = 0;	// keine Taste gedrckt
	
		return button;
	
	sei();

}

/*
uint8_t read_LUT(uint16_t adc_val_button)
{
	uint8_t button;
			
	// LUT fr AREF = AVCC = U_teiler = 4,93V
		
		
		if(adc_val_button <= 324) button = 1;	// Taster 1
		if(adc_val_button <= 274) button = 2;
		if(adc_val_button <= 219) button = 3;
		if(adc_val_button <= 163) button = 4;
		if(adc_val_button <= 105) button = 5;	// Taster 5
		if(adc_val_button <=  44) button = 0;	// keine Taste gedrckt
	
		return button;

}
*/


/***************************************************************************************
*
*Funktionsname:		my_ultoa
*
*Rckgabe:			keine
*
*Parameter:			32bit-uint-Zahl / String
*
*Aufgabe:			Wandelt eine uint-32bit Zahl mit fhrenden Nullen in einen String
*
*                           |
*
*getestet: funktioniert
***************************************************************************************/

void my_ultoa(uint32_t zahl, char* string)
{
	int8_t i;                             // schleifenzhler
 
	string[10]='\0';                       // String Terminator
	for(i=9; i>=0; i--) 
	{							// Modulo rechnen, dann den ASCII-Code von '0' addieren
		string[i]=(zahl % 10) +'0';         
		zahl /= 10;
	}
}






/***************************************************************************************
*
*Funktionsname:		round_32bit
*
*Rckgabe:			keine
*
*Parameter:			string von 32bit-Zahl / Index fr Rundung relevante Ziffer
*
*Aufgabe:		Rundet eine ** vorzeichenlose ** 32bit Zahl in 11 Byte String 
*											oder
*				Rundet eine ** vorzeichenbehaftete ** 32bit Zahl in 12 Byte String				
*					
*Bsp. uint32_t in 11Byte String:				
*                           |
*		Index	0 1 2 3 4 5 6 7 8 9 10    (Byte)
*				
*	32bit-Zahl	0 0 1 5 7 8 9 9 6 5 /0    (String)
*
*				0 0 1 5,7 8 9 9 6 5 /0 ....Zahl mit gedachtem Komma und 2 
*                           ^										Nachkommastellen
*	                        | 	Index fr Rundung relavante Ziffer = 6	
*
* ->  round_32bit(string,6);
*
*getestet: funktioniert
***************************************************************************************/

void round_32bit(char* string , uint8_t index_digit)
{
	
	uint8_t i , index_end;		
	
	
	if( (string[0] == '-') || (string[0] == ' ') )
	{								// vorzeichenbehaftete Zahl? (12Byte String)
		string+1;					// 1. Byte = sign -> Pointer auf 2tes Byte stellen
		index_digit-1;				// Index wrde sich mitverschieben, daher korrektur
		index_end = 10;				// index_end fr 12 Byte string
	}
	else
		index_end = 9;				// index_end fr 11 Byte String
	
	
	if (string[index_digit] >= '5')			// mu aufgerundet werden ?
	{
		
		for (i= index_digit-1 ; i>=0 ; i--)	// einzelne Ziffern ansprechen
		{
			
			string[i] += 1;					// Ziffer aufrunden
			if (string[i] <= '9')			// wenn kein bertrag Schleife verlassen
				break;
			
			else
				string[i] = '0';			// bertrag + berlauf
		}
		
	} 
	
	for (i=index_digit ; i<=index_end ; i++)		// gerundete Stellen auf Null setzen
		string[i] = '0';	
	
}





/***************************************************************************************
*
*Funktionsname:		print_lcd_uint
*
*Rckgabe:			keine
*
*Parameter:			String von  **uint** Zahl (egal ob von 8, 16 oder 32bit Zahl)
*					Index von 1. Vorkommastelle
*					Index von 1. Nachkommastelle
*					Anzahl Nachkommastellen
*
*Aufgabe:			Gibt formatierte Zahlen (uint) mit Komma am LCD aus
*					Erfordert ein 11 Byte groes Feld !!
*
*                   |   |
Bsp.:	Index	0 1 2 3 4 5 6 7 8 9 10     (Byte)
*				
*	32bit-Zahl	0 0 1 5 7 9 0 0 0 0  /0    (String, Zahl wurde vorher gerundet!)
*
*				0 0 1 5,7 9 0 0 0 0  /0 ....Zahl mit gedachtem Komma und 2 
*                   ^   ^				  					Nachkommastellen
*	                |   | Index 1. Nachkommastelle = 4
*                   |
*                   | Index 1. Vorkommastelle = 2	
*
*->	print_lcd_uint (string,2,4,2);
*
* getestet: funktioniert
***************************************************************************************/

void print_lcd_uint(char* string , uint8_t index_start , uint8_t index_digit_1 , uint8_t frac)
{
	uint8_t i;
	uint8_t zero_flag = 1;					// Statusbit fr vorangestellte Nullen
			
	for(i=index_start ; i<index_digit_1 ; i++)	// von 1. Vorkomma-Stelle bis letzte
	{											// Vorkomma-Stelle
		
		if (zero_flag == 0 || string[i] != '0') // Ziffer ungleich Null oder z_flag=0
		{
			
			lcd_putc(string[i]);			// Ziffer an LCD ausgeben
			zero_flag = 0;					// keine vorgestellten Nullen
		} 
		
		else
			lcd_putc('0');
		
		/*
		if ( (i==index_digit_1-1) && (string[i]=='0') )
			lcd_putc('0');
		
		if ( (i!=index_digit_1-1) && (string[i] == '0') )
			lcd_putc(' ');					// Null da, Platz freilassen
		*/	
			
	}	
	
	lcd_putc(',');							// Komma ausgeben
	
	for(; i<(index_digit_1 + frac) ; i++)   // von 1. Nachkommastelle bis letzte
		lcd_putc(string[i]);				// Nachkommastellen ausgeben
}





/***************************************************************************************
*
*Funktionsname:		display_lcd_page1
*
*Rckgabe:			keine
*
*Parameter:			keine
*
*Aufgabe:			Gibt die 1. Anzeige-Seite am Display aus
*
*
* getestet: 
***************************************************************************************/

void display_lcd_page1(void)
{
	//lcd_clrscr();							// Display lschen und cursor home
	lcd_gotoxy(0,0);
	
	my_ultoa(voltage_electrodes , string);	// Umwandlung in String
	round_32bit(string , 8);				// auf 1ne Nachkommastelle runden
	print_lcd_uint(string , 5 , 7 , 1);		// Spannung formatiert auf LCD ausgeben
	lcd_putc('V');							// Volt Symbol ausgeben
	
	lcd_putc(' ');							// Leerstelle
	
	my_ultoa(current_electrodes , string);	// Umwandlung in String
	round_32bit(string , 6);				// auf 2 Nachkommastellen runden
	print_lcd_uint(string , 2 , 4 , 2);		// Strom formatiert auf LCD ausgeben
	lcd_puts("mA");							// Strom Symbol ausgeben
	
	lcd_putc(' ');							// Leerstelle
		
	if (current_mode == 0)					// Stromart anzeigen
		lcd_puts("DC");	
	else
		lcd_puts("AC");	
		
	lcd_gotoxy(0, 1);						// nchste Zeile, 1 Zeichen
	
	itoa(hour_countdown, string, 10);		// Stunden in ASCII umwandeln
	if(hour_countdown < 10)					// wenn Stunden kleiner 10
		lcd_putc('0');						// Null ausgeben
	lcd_puts(string);						// Stunden ausgeben
	
	lcd_putc(':');							// Doppelpunkt ausgeben
	
	itoa(minute_countdown, string, 10);		// Minuten in ASCII umwandeln
	if(minute_countdown < 10)				// wenn Minuten kleiner 10
		lcd_putc('0');						// Null ausgeben
	lcd_puts(string);						// Minuten ausgeben
	
	lcd_putc(':');							// Doppelpunkt ausgeben	
	
	itoa(second_countdown, string, 10);		// Sekunden in ASCII umwandeln
	if(second_countdown < 10)				// wenn Sekunden kleiner 10
		lcd_putc('0');						// Null ausgeben
	lcd_puts(string);						// Sekunden ausgeben
	
	lcd_puts("   ");						// 3 Leerstellen
	
	itoa(ac_period, string, 10);			// AC-Periodendauer in ASCII umwandeln
	if(ac_period < 10)						// wenn Sekunden kleiner 10
		lcd_putc('0');						// Null ausgeben
	lcd_puts(string);						// Sekunden ausgeben
	lcd_puts("sec");						// Symbol fr Sekunden ausgeben
	
}		
	
	

/***************************************************************************************
*
*Funktionsname:		countdown_reset
*
*Rckgabe:			keine
*
*Parameter:			keine
*
*Aufgabe:			Reset Variablen fr Countdown
*
*
* getestet: 
***************************************************************************************/

void countdown_reset(void)
{
	second_countdown = 2;			// Variablen fr Countdown zurcksetzen
	hour_countdown = 0;
	minute_countdown = 0;	
}






/***************************************************************************************
*
*Funktionsname:		countdown
*
*Rckgabe:			keine
*
*Parameter:			keine
*
*Aufgabe:			pro Aufruf wird 1ne Sekunde heruntergezhlt und bei 0 Gert 
*					ausgeschaltet
*
* getestet: 
***************************************************************************************/	
	
void countdown(void)
{	
	second_countdown--;				// Sekunde -1
			
	if( second_countdown == -1)		// wenn Sekunden abgelaufen (0 entspricht 60)
	{								// Abgeschaltet wird nach Ablauf der Sekunde 0 !
				
		if( (minute_countdown > 0) || (hour_countdown > 0) )
		{											// wenn Restzeit vorhanden	
			second_countdown = 59;					// Sekunden wieder auf 59
			minute_countdown--;						// Minuten -1
				
		}
			
		if( (minute_countdown == -1) && (hour_countdown > 0) )	
		{							// wenn Minuten abgelaufen und Restzeit da
			minute_countdown = 59;						// Minuten wieder auf 59
			hour_countdown--;							// Stunden -1
		}
			
			
		if( (minute_countdown == 0) && (hour_countdown == 0) )
		{												// keine Restzeit mehr
			gen_on = 0;									// Gert ausschalten
			countdown_reset();							// countdown reset
		}		
	}
}


